import datetime
import time


class Maintenance(object):

    def __init__(self, client, json=False, obj=False):
        self.client = client
        self._id = False
        if json:
            self.from_json(json)
        elif obj:
            self.from_obj(obj)
        else:
            raise "Missing definition: use json or obj parameter"

    def __repr__(self):
        checks = []
        for check in self.checks:
            if check:
                checks.append(check.name)
            else:
                checks.append("<deleted check>")
        return """
        pingdom.Maintenance <{0}>
         name: {1}
         start: {2}
         stop: {3}
         checks: {4}
        """.format(self._id,
                   self.name,
                   self.start,
                   self.stop,
                   ", ".join(checks))

    def to_json(self):
        check_ids = [str(check._id) for check in self.checks]
        data = {
            # "__csrf_magic": "",
            # "id": "",
            "description": self.name,
            "from-date": "{0}.{1}.{2}.".format(self.start.year, self.start.month, self.start.day),
            "from-time": "{0}:{1}".format(self.start.hour, self.start.minute),
            "to-date": "{0}.{1}.{2}.".format(self.stop.year, self.stop.month, self.stop.day),
            "to-time": "{0}:{1}".format(self.stop.hour, self.stop.minute),
            "start": int(time.mktime(self.start.timetuple())),
            "stop": int(time.mktime(self.stop.timetuple())),
            "checks": "[{0}]".format(",".join(check_ids))
        }
        return data

    def from_json(self, obj):
        self._id = int(obj['id'])
        self.name = obj["description"]
        self.start = datetime.datetime.fromtimestamp(obj['start'])
        self.stop = datetime.datetime.fromtimestamp(obj['end'])
        self.checks = [self.client.get_check(_id=int(x['compound_id'])) for x in obj['checks']]

    def from_obj(self, obj):
        self.name = obj["name"]
        self.start = obj['start']
        self.stop = obj['stop']
        self.checks = obj['checks']



import requests
from requests.auth import HTTPBasicAuth
import json


class ApiError(Exception):

    def __init__(self, http_response):
        content = json.loads(http_response.content)
        self.status_code = http_response.status_code
        self.status_desc = content['error']['statusdesc']
        self.error_message = content['error']['errormessage']
        super(ApiError, self).__init__(self.__str__())

    def __repr__(self):
        return 'pingdom.ApiError: HTTP `%s - %s` returned with message, "%s"' % \
               (self.status_code, self.status_desc, self.error_message)

    def __str__(self):
        return self.__repr__()


class Api(object):

    def __init__(self, username, password, apikey, email=False, version="2.0"):
        self.base_url = "https://api.pingdom.com/api/" + version + "/"
        self.auth = HTTPBasicAuth(username, password)
        self.headers = {'App-Key': apikey}
        if email:
            self.headers['Account-Email'] = email

    def send(self, method, resource, resource_id="", data={}, params={}):
        response = requests.request(method, self.base_url + resource + "/" + str(resource_id), auth=self.auth, headers=self.headers, data=data, params=params)
        if response.status_code != 200:
            raise ApiError(response)
        else:
            return response.json()



class Check(object):

    SKIP_ON_PRINT = ["cached_definition", "_id", "api"]
    SKIP_ON_JSON = [
        "api", "alert_policy_name", "cached_definition", "created", "lastresponsetime",
        "lasttesttime", "_id", "id", "status"
    ]

    def __init__(self, api, name, json=False, obj=False):
        self.api = api
        self.name = name
        if json:
            self.from_json(json)
        elif obj:
            self.from_obj(obj)
        else:
            raise "Missing check definition: use json or obj parameter"

    def __repr__(self):
        attr = ["{0}: {1} ".format(k, v) for k, v in self.__dict__.items() if k not in self.SKIP_ON_PRINT]
        return "\npingdom.Check <%s> \n  %s" % (self._id, "\n  ".join(attr))

    def to_json(self):
        obj = {}
        for k, v in self.__dict__.items():
            if k in self.SKIP_ON_JSON:
                pass
            elif k == "tags":
                obj["tags"] = ",".join(v)
            elif k == "requestheaders":
                i = 0
                for x, y in v.items():
                    obj["requestheader" + str(i)] = x + ":" + y
                    i = i + 1
            else:
                obj[k] = v
        return obj

    def from_json(self, obj):
        for k, v in obj.items():
            if k == "tags":
                self.tags = [x["name"] for x in v]
            if k == "name":
                self.name = v.lower()
            elif k == "hostname":
                self.host = v
            elif k == "id":
                self._id = v
            elif k == "status":
                if v == "paused":
                    self.paused = True
                else:
                    self.paused = False
            elif k == "type" and type(k) is dict:
                self.type = v.keys()[0]
                for x, y in v[self.type].items():
                    setattr(self, x, y)
            else:
                setattr(self, k, v)

    def from_obj(self, obj):
        for k, v in obj.items():
            setattr(self, k, v)

    def fetch(self):
        res = self.api.send('get', "checks", self._id)['check']
        self.from_json(res)
